/*
 * Copyright (C) 2011-2012 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// iCEDIP24           pll_test        iCEDIP24
//  37            1            24  VCC  
//  43            2            23  VDAC0   36
//  44            3            22  VDAC1   35
//  45            4            21  VDAC2   34
//  46            5            20  VDAC3   32
//  47            6            19          31
//  48  INV1_IN   7            18          26
//   2  INV1_OUT  8            17          25
//   3  INV2_IN   9            16  DAC0    21
//   4  INV2_OUT  10           15  DAC1    20
//   9  PLLCTRL   11           14  DAC2    19
//           GND  12           13  DAC3    18
	   

module ntsc_test (
    input CLK14M             // 14.314 MHz
    , output VDAC[3:0]
   );

   wire DISP;
   wire Y_DISP;
   wire FLAME;
   wire [3:0] NTSC;
   ntsc_timing nt_inpl (.NTSC_CLK(CLK14M), .NTSC_OUT(NTSC)
      , .XDISP(X_DISP), .YDISP(Y_DISP), .FLAME(FLAME));

   reg [3:0] v_dac;
   reg [9:0] x;
   reg [8:0] y;
   reg d_prev;
   reg y_prev;
   //assign VDAC = NTSC[3:0];
   assign VDAC = (X_DISP && Y_DISP) ? v_dac[3:0] : NTSC[3:0];


always @(posedge CLK14M) begin
    d_prev <= X_DISP;
    if (X_DISP) begin
        y_prev <= Y_DISP;
        if (Y_DISP) begin
           if (y_prev != Y_DISP) begin
              y <= 0;
           end else if (d_prev != X_DISP) begin
              y <= y + 1;
              x <= 0;
           end else 
              x <= x+1;
           v_dac <= FLAME ? 11 : 4;
       end
    end 
end

endmodule

module ntsc_encoder (
    input NTSC_CLK            // 14.314 MHz
    , output NTSC_OUT[3:0]
    , output XDISP
    , output YDISP
    , output FLAME
   );

reg [9:0] x; // 14.314 MHz counter
reg [8:0] y; 
reg [1:0] f;
reg [3:0] state;
reg r_disp;

assign NTSC_OUT = state[3:0];
assign XDISP = r_disp;
assign YDISP = (27 <= y) && (y < 27 + 228);
assign FLAME = f[0];
always @(posedge NTSC_CLK) begin
    if ((~f[0] && (y == 262 -1) &&(x < 455 -1))
       || (~f[0] && (y != 262 -1) &&(x < 910 -1))
       || (f[0] &&(x < 910 -1))) begin // 227.5fs
        x <= x + 1; 
        r_disp <= ((39 + 14)*4 -1 <= x) && (x < (39 +14 +160)*4 -1); 
    end else begin
        if (y < 262) begin 
            x <= 0;
            y <= y +1;
            r_disp <= 1'b0; 
        end else if (~f[0] && (y == 9)) begin
            x <= 455;
            y <= 0;
            f <= f + 1;
            r_disp <= 1'b0; 
        end else begin 
            x <= 0;
            y <= 0;
            f <= f + 1;
            r_disp <= 1'b0;  
       end
    end
end

always @(posedge NTSC_CLK) begin
    if ((3 <= y) && (y < 6)) begin
        if (x < 455 - 67) state <= 0; // -40IRE
        else if ((455 <= x) && (x < 910 - 67)) state <= 0;     // -40IRE
        else state <= 4;   // 0IRE
    end else if (y < 9) begin
        if (x < 33) state <= 0; // -40IRE
        else if ((455 < x) && (x < 455 + 33)) state <= 0;     // -40IRE
        else state <= 4;   // 0IRE
    end else if (f[0] && (y < 10)) begin
        state <= 4;   // 0IRE
    end else begin
        if (x < 6*4) state <= 4; // 0IRE
        // H pluse
        else if (x < (6+17)*4) state <= 0;     // -40IRE
        else if (x < (6+17+2)*4) state <= 4;   // 0IRE
        // color burst
        else if (x < (6+17+2)*4+2) state <= 5;  // 10IRE
        else if (x < (6+17+2)*4+4) state <= 2;  // -20IRE
        else if (x < (6+17+2)*4+6) state <= 6;  // 20IRE
        else if (x < (6+17+2)*4+8) state <= 2;  // -20IRE
        else if (x < (6+17+2)*4+10) state <= 6;  // 20IRE
        else if (x < (6+17+2)*4+12) state <= 2;  // -20IRE
        else if (x < (6+17+2)*4+14) state <= 6;  // 20IRE
        else if (x < (6+17+2)*4+16) state <= 2;  // -20IRE
        else if (x < (6+17+2)*4+18) state <= 6;  // 20IRE
        else if (x < (6+17+2)*4+20) state <= 2;  // -20IRE
        else if (x < (6+17+2)*4+22) state <= 6;  // 20IRE
        else if (x < (6+17+2)*4+24) state <= 2;  // -20IRE
        else if (x < (6+17+2)*4+26) state <= 6;  // 20IRE
        else if (x < (6+17+2)*4+28) state <= 2;  // -20IRE
        else if (x < (6+17+2)*4+30) state <= 6;  // 20IRE
        else if (x < (6+17+2)*4+32) state <= 2;  // -20IRE
        else if (x < (6+17+2)*4+34) state <= 6;  // 20IRE
        else if (x < (6+17+2)*4+36) state <= 3;  // -10IRE

        else if (x < (6+17+2+9+5)*4) state <= 4;  // 0IRRE
        
        else state <= 4; // 0IRE
    end
end
endmodule
