/*
 * Copyright (C) 2011-2012 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// iCEDIP24           pll_test        iCEDIP24
//  37            1            24  VCC  
//  43            2            23  VDAC0   36
//  44            3            22  VDAC1   35
//  45            4            21  VDAC2   34
//  46            5            20  VDAC3   32
//  47            6            19          31
//  48  INV1_IN   7            18          26
//   2  INV1_OUT  8            17          25
//   3  INV2_IN   9            16  DAC0    21
//   4  INV2_OUT  10           15  DAC1    20
//   9  PLLCTRL   11           14  DAC2    19
//           GND  12           13  DAC3    18
	   

module ext_pll (
    input INV1_IN
   ,output INV1_OUT
   ,input INV2_IN
   ,output INV2_OUT
   ,input REFCLK
   ,output PLLCTRL
   );
   assign INV1_OUT = ~INV1_IN;
   assign INV2_OUT = ~INV2_IN;
   reg [10:0] ref_counter;
   reg ref_out;
   reg [9:0] pll_counter;
   reg pll_out;

    always @(posedge REFCLK)
    begin
            if (ref_counter == 1623 -1)
               begin
                 ref_counter <= 0;
                 ref_out <= ~ref_out;
               end
            else
                 ref_counter <= ref_counter + 1;
    end
    always @(posedge INV2_IN)
    begin
            if (pll_counter == 1024 -1)
               begin
                 pll_counter <= 0;
                 pll_out <= ~pll_out;
               end
            else
                 pll_counter <= pll_counter + 1;
    end

   simple_pll pll_inpl (.PLLCLK(pll_out), .REFCLK(ref_out), .PLLCTRL(PLLCTRL));
endmodule

`define TEST1
module simple_pll (
		input  PLLCLK     // DATA IN
		, input REFCLK    // DATA OUT
		, output PLLCTRL  //
		);
    reg up;
    reg down;
    wire up_out = down & ~up;
    wire down_out = up & ~down;
    assign PLLCTRL = up_out ? 1'b1 : down_out ? 1'b0 : 1'bz ;

`ifdef TEST1
    always @(posedge PLLCLK)
    begin
            down <= (down & up);
    end
    always @(posedge REFCLK)
    begin
            up <= (down & up);
    end
`else
    always @(posedge PLLCLK , posedge REFCLK)
    begin
       if (down & up)
           begin
             down <= 1'b0 ;
             up <= 1'b0;
           end
       else 
           begin
             if (~PLLCLK) down <= 1'b1;
             if (~REFCLK) up <= 1'b1;
           end       
    end
`endif
endmodule
